
#ifndef BOOT_H
#define BOOT_H

/** I N C L U D E S **********************************************************/
#include "system\typedefs.h"


/** D E F I N I T I O N S ****************************************************/

/* Bootloader Version */
#define MINOR_VERSION   0x01    // Bootloader Version
#define MAJOR_VERSION   0x02

/* State Machine */
#define WAIT_FOR_CMD    0x00    // Wait for Command packet
#define SENDING_RESP    0x01    // Sending Response


/** S T R U C T U R E S ******************************************************/

/*********************************************************************
 * General Data Packet Structure:
 * ACCESS RAM: Buffer Starting Address is assigned by compiler.
 * __________________           DATA_PACKET.FIELD
 * |    COMMAND     |   0       [CMD]
 * |      LEN       |   1       [LEN]
 * |     ADDRL      |   2       [        ]  [ADR.LOW]
 * |     ADDRH      |   3       [ADR.pAdr]: [ADR.HIGH]
 * |     ADDRU      |   4       [        ]  [ADR.UPPER]
 * |                |   5       [DATA]
 * |                |
 * .      DATA      .
 * .                .
 * |                |   62
 * |________________|   63
 *
 ********************************************************************/
#define EP1_BUF_SIZE 64

#define OVER_HEAD   5           // Overhead: <CMD_CODE><LEN><ADDR:3>
#define DATA_SIZE   (EP1_BUF_SIZE - OVER_HEAD)

typedef union _BOOT_DATA_PACKET
{
    char _byte[EP1_BUF_SIZE];
    struct
    {
        enum
        {
            READ_SW3    = 'a',
            LEDS_ON     = 'b',
            LEDS_OFF    = 'c',
            RESET       = 0xFF
        }CMD;                           // <COMMAND>
        char len;                       // <LEN>
        union
        {
            rom far char *pAdr;         // Address Pointer
            struct
            {
                char low;               // <ADDRL>  (little-endian)
                char high;              // <ADDRH>
                char upper;             // <ADDRU>
            };
        }ADR;
        char data[DATA_SIZE];
    };
} BOOT_DATA_PACKET;


/** B O O T L O A D E R  C O M M A N D  B U F F E R **************************/

extern volatile far BOOT_DATA_PACKET dataPacket;


/** E X T E R N S ************************************************************/



/** P U B L I C  P R O T O T Y P E S *****************************************/

void BootService(void);


#endif //BOOT_H
